<?php
/**
 * Plugin cilien
 * Copyright (c) Christophe IMBERTI
 * Licence Creative commons by-nc-sa
 */
 
if (!defined('_ECRIRE_INC_VERSION')) return;

// Pour contourner une limite de SPIP 2.1 que j'ai signalee (http://core.spip.org/issues/3116)
// et qui a ete corrigee dans SPIP 3.0.14 mais pas dans SPIP 2.1
define('_CILIEN_EXTRAIRE_DOMAINE', '/^(?:[^\W_]((?:[^\W_]|-){0,61}[^\W_,])?\.)+[a-z0-9]{2,6}\b/Si');

include_spip('inc/lien');
include_spip('inc/filtres');
include_spip('inc/texte');

function cilien_traite_objet($objet,$id_objet,$champs){

	if ($objet AND $id_objet AND is_array($champs)){
	
		$letexte = '';
		// concatener les champs
		foreach ($champs AS $cle=>$valeur){
			if (is_string($valeur)){
				// article virtuel
				if ($cle=='chapo' AND $valeur AND substr($valeur,0,1)=='=')
					$valeur = substr($valeur,1);
				
				$letexte .= $valeur.' ';
			}
		}
	
		// marquer tous ses liens existants comme obsolete
		sql_updateq("spip_cilien", array('obsolete'=>'oui'), "objet='".$objet."' AND id_objet=$id_objet");
	
		// passer le contenu dans expanser_liens pour transformer les liens
		// Pour contourner une limite de SPIP 2.1 que j'ai signalee (http://core.spip.org/issues/3116)
		// et qui a ete corrigee dans SPIP 3.0.14 mais pas dans SPIP 2.1
		cilien_extrait_liens($objet,$id_objet,expanser_liens($letexte));

		// supprimer les liens obsoletes de l'objet / id_objet
		sql_delete("spip_cilien", "objet='".$objet."' AND id_objet=".$id_objet." AND obsolete='oui'" );
	}
}


function cilien_extrait_liens($objet,$id_objet,$texte=''){
	
	if ($texte AND strstr($texte, 'href=')){	// evite le preg_match_all si inutile
		if (preg_match_all(_EXTRAIRE_LIENS,$texte, $regs, PREG_SET_ORDER)){
			foreach ($regs as $r) {
				$tout = $r[0];
				
				$url ='';
				$l = '';
				if ($tout AND strstr($tout, 'href=')){
					if (preg_match("/href='([^']*)'/",$tout, $result))
						$l = $result[1];
					elseif (preg_match('/href="([^"]*)"/',$tout, $result))
						$l = $result[1];
				}
	
				if ($l){
					// Enlever le slash de fin
                                        // sauf dans le cas d'une URL qui a plusieurs slash (en dehors de ://)
					if (substr($l,-1)=='/' AND substr_count(str_replace('//','',$l), '/') <= 1) {
						$l = substr($l,0,-1);
                                        }
                                        
					// Ne pas memoriser les url de l'espace prive
					// Sur [lien->art8] SPIP 2.1 met une URL absolue dans l'espace prive
					// Sur <doc18> SPIP 2.1 met une URL relative avec ../IMG/ dans l'espace prive
                                        $ci_dir_racine = substr(_DIR_IMG,0,-strlen(_NOM_PERMANENTS_ACCESSIBLES));
					if (strpos($l,_DIR_RESTREINT_ABS)===FALSE AND substr($l,0,strlen($ci_dir_racine))!=$ci_dir_racine) {
						
						// Reperer le protocole http
						// Si pas de protocole de lien -> lien relatif
						if (preg_match(',^(https?):/*,S', $l, $m)) {
							$l = substr($l, strlen($m[0]));
							$protocol = $m[1];
	
							// valider le nom de domaine
							if (preg_match(_CILIEN_EXTRAIRE_DOMAINE, $l)){
								// supprimer les ponctuations a la fin d'une URL
								preg_match('/^(.*?)([,.;?]?)$/', $l, $k);
								if (isset($k[1])  AND $k[1])
									$url = $protocol.'://'.$k[1];
								else
									$url = $protocol.'://'.$l;
							}
						} else {
							// precaution
							if (substr($l, 0, 3)=="../")
								$l = substr($l,3);
							
							// eviter les autres protocoles (mailto, etc.) et les ancres
							if (strpos(substr($l, 0, 12),':')===FALSE AND substr($l, 0, 1)!="#")
								$url = $l;
						}
					}
				}
				
				// Memoriser l'URL
				if ($url)
					cilien_enregistre_lien($objet,$id_objet,$url);
			
			}
		}
	}
}


function cilien_enregistre_lien($objet,$id_objet,$url){
	$id_objet = intval($id_objet);
	
	if ($objet AND $id_objet AND $url){		
		// regarder si le lien est deja reference pour cet objet et le creer eventuellement
		if ($row = sql_fetsel("*", "spip_cilien", "url=".sql_quote($url)." AND objet='".$objet."' AND id_objet=".$id_objet))
			sql_updateq("spip_cilien", array('maj'=>'NOW()', 'obsolete'=>'non'), "url=".sql_quote($row['url'])." AND objet='".$row['objet']."' AND id_objet=".$row['id_objet']);
		else
			$id_lien = sql_insertq("spip_cilien",array('objet' => $objet, 'id_objet' => $id_objet, 'url' => $url, 'obsolete' => 'non'));		
	}
}

// Recenser les liens par tranche
function cilien_traite_tranche($type_pas='', $pas='', $timeout=''){
	$cibreak = false;
	if ($type_pas=='grandpas'){
            if (!$pas)
                $pas = 500;
            // duree maximum en secondes
            if ($timeout)
                $dureemax = $timeout - 3;
            else
                $dureemax = 10;
	} else {
            if (!$pas)
                $pas = 50;
            // duree maximum en secondes
            if ($timeout)
                $dureemax = $timeout - 3;
            else
                $dureemax = 3;
       }	

       
	$time = time();

	if (!sql_count(sql_select("*", "spip_cilien_tranche"))){
		sql_insertq("spip_cilien_tranche",array('table_objet' => 'spip_articles', 'objet' => 'article', 'date_tranche' => ''));
		sql_insertq("spip_cilien_tranche",array('table_objet' => 'spip_rubriques', 'objet' => 'rubrique', 'date_tranche' => ''));
	}

	$tranches = sql_allfetsel("*","spip_cilien_tranche");

	foreach($tranches as $tranche) {
		if (!$cibreak){
			$table = $tranche['table_objet'];
			$objet = $tranche['objet'];
			$col_id = id_table_objet($table);		
			$where = '';
                        $orderby = array('maj',$col_id);
                        
			$tranche_inf = $tranche['date_tranche'];
			$tranche_sup = $tranche_inf;
                        $id_inf = cilien_lire_dernier_id($objet,$tranche_inf);
			$id_sup = $id_inf;

			if ($tranche_inf){
                            $where = "maj > '".date("Y-m-d H:i:s",strtotime($tranche_inf))."'";
                            
                            // cas d'un site issue d'une fusion
                            // avec plus de 500 enregistrements ayant exactement la meme date de maj
                            if ($id_inf){
                                $where_maj_et_id = "maj='".date("Y-m-d H:i:s",strtotime($tranche_inf))."' AND ".$col_id.">".$id_inf;
                                if (sql_countsel($table, $where_maj_et_id)>=1) {
                                    // il reste des enregistrements a traiter sur cette date de maj
                                    $where = $where_maj_et_id;                                
                                }	
                            }
                        }
                        
			// on prend au plus $pas enregistrement a partir de $tranche_inf
                        $compte=0;
			$res = sql_select("*",$table,$where, '', $orderby, '0,'.$pas);
			while ($row = sql_fetch($res) AND !$cibreak){
				cilien_traite_objet($objet,$row[$col_id],$row);
				$tranche_sup = $row['maj'];
				$id_sup = $row[$col_id];
                                $compte++;
                                
				// tranche auto ajustable
				// si plus de dureemax secondes on sort de la boucle
				$n = time() - $time;
				if ($n > $dureemax){
					$cibreak = true;
				}
			}

                        if ($compte>=1){
//                            spip_log("CILIEN : ".$objet.':'.$compte,_LOG_ERREUR);	
                            if ($row = sql_fetsel("*", "spip_cilien_tranche", "table_objet='".$table."'"))
                                    sql_updateq("spip_cilien_tranche", array('date_tranche'=>$tranche_sup), "table_objet='".$table."'");
                            else
                                    $return = sql_insertq("spip_cilien_tranche",array('table_objet' => $table, 'objet' => $objet, 'date_tranche' => $tranche_sup));

                            // cas d'un site issue d'une fusion
                            // avec plus de 500 enregistrements ayant exactement la meme date de maj
                            cilien_ecrire_date_et_id($objet,$tranche_sup,$id_sup);
                        }
		}
	}	
}

// Pour contourner une limite de SPIP 2.1 que j'ai signalee (http://core.spip.org/issues/3116)
// et qui a ete corrigee dans SPIP 3.0.14 mais pas dans SPIP 2.1
function cilien_expanser_liens($texte, $connect='') {

	$texte = traiter_raccourci_ancre(traiter_raccourci_glossaire(cilien_traiter_raccourci_liens($texte)));	
	$sources = $inserts = $regs = array();
	if (preg_match_all(_RACCOURCI_LIEN, $texte, $regs, PREG_SET_ORDER)) {
		$lien = charger_fonction('lien', 'inc');
		foreach ($regs as $k => $reg) {

			$inserts[$k] = '@@SPIP_ECHAPPE_LIEN_' . $k . '@@';
			$sources[$k] = $reg[0];
			$texte = str_replace($sources[$k], $inserts[$k], $texte);

			list($titre, $bulle, $hlang) = traiter_raccourci_lien_atts($reg[1]);
			$r = $reg[count($reg)-1];
			// la mise en lien automatique est passee par la a tort !
			// corrigeons pour eviter d'avoir un <a...> dans un href...
			if (strncmp($r,'<a',2)==0){
				$href = extraire_attribut($r, 'href');
				// remplacons dans la source qui peut etre reinjectee dans les arguments
				// d'un modele
				$sources[$k] = str_replace($r,$href,$sources[$k]);
				// et prenons le href comme la vraie url a linker
				$r = $href;
			}
			$regs[$k] = $lien($r, $titre, '', $bulle, $hlang, '', $connect);
		}
	}

	// on passe a traiter_modeles la liste des liens reperes pour lui permettre
	// de remettre le texte d'origine dans les parametres du modele
	$texte = traiter_modeles($texte, false, false, $connect, array($inserts, $sources));
 	$texte = corriger_typo($texte);
	$texte = str_replace($inserts, $regs, $texte);
	
	return $texte;
}

function cilien_traiter_raccourci_liens($t) {
	return preg_replace_callback(_EXTRAIRE_LIENS, 'cilien_traiter_autoliens', $t);
}

function cilien_traiter_autoliens($r) {
	
	if (count($r)<2) return reset($r);
	list($tout, $l) = $r;
	if (!$l) return $tout;
	// reperer le protocole
	if (preg_match(',^(https?):/*,S', $l, $m)) {
		$l = substr($l, strlen($m[0]));
		$protocol = $m[1];
	} else 	$protocol = 'http';
	// valider le nom de domaine
	if (!preg_match(_CILIEN_EXTRAIRE_DOMAINE, $l)) return $tout;
	// supprimer les ponctuations a la fin d'une URL
	preg_match('/^(.*?)([,.;?]?)$/', $l, $k);
	$url = $protocol.'://'.$k[1];
	$lien = charger_fonction('lien', 'inc');
	$r = $lien($url,'','','','','nofollow') . $k[2];
	// si l'original ne contenait pas le 'http:' on le supprime du clic
	return $m ? $r : str_replace('>http://', '>', $r);
}

function cilien_avancement() {
	$return = array();

	$tranches = sql_allfetsel("*","spip_cilien_tranche");

	if (!$tranches)
		$tranches = array(
			array('table_objet'=>'spip_articles','objet'=>'article','date_tranche'=>''),
			array('table_objet'=>'spip_rubriques','objet'=>'rubrique','date_tranche'=>'')
			);
	
	foreach($tranches as $tranche){
		$table = $tranche['table_objet'];
		$objet = $tranche['objet'];
		$tranche_inf = $tranche['date_tranche'];
                $id_inf = cilien_lire_dernier_id($objet,$tranche_inf);

		if ($tranche_inf){
			$where = "maj > '".date("Y-m-d H:i:s",strtotime($tranche_inf))."'";                        
			$reste = sql_countsel($table,$where);
			$total = sql_countsel($table);
			$return[$objet] = 100*($total-$reste)/$total;
		} else {
			$return[$objet] = 0;
		}
	}
	
	return $return;
}


function cilien_remplace_dans_objet($lieninitial,$lienfutur,$objet,$id_objet){
    $id_objet = intval($id_objet);
    $return = 0;
    
    if ($objet AND $id_objet){
        $table = 'spip_articles';
        $primary = 'id_article';
        $select = 'texte,descriptif,virtuel,titre';
        
        if ($objet=='rubrique'){
            $table = 'spip_rubriques';
            $primary = 'id_rubrique';
            $select = 'texte,descriptif,titre';
        }
        
        $tableau_des_remplacements = array();
        
        if ($champs = sql_fetsel($select, $table, $primary."=".$id_objet)){
            foreach ($champs AS $cle=>$valeur){
                // le champ contient le lien initial ?
                if (strpos($valeur,$lieninitial) !== false){
                    // noter les remplacement a faire
                    $tableau_des_remplacements[$cle] = str_replace($lieninitial,$lienfutur,$valeur);
                }
            }
            
            // remplacer
            if (count($tableau_des_remplacements)>=1){
                sql_updateq($table, $tableau_des_remplacements, $primary."=".$id_objet);
                $return = 1;
            }
        }
    }
    
    return $return;
}


function cilien_verifier_url($url){
    $return = false;
    
    // Ajouter http:// si necessaire
    if (!tester_url_absolue($url)) {
            $url = 'http://' . $url;
    } elseif (strncmp($url, "//", 2) == 0) {
            $url = 'http:' . $url;
    }
    
    // Enlever les caracteres illegaux
    $safe_url = filter_var($url, FILTER_SANITIZE_URL);
    
    if ($safe_url == $url) {
        // Verifier l'url
        if (!filter_var($safe_url, FILTER_VALIDATE_URL) === false)
            $return = true;
    }
    return $return;
}


function cilien_verifier_domaine($domaine){
    $return = false;

    $tscs = str_replace(array('.','-','/','_'),'',$domaine);
    if (ctype_alnum($tscs))
            $return = true;

    return $return;
}

function cilien_lire_dernier_id($objet,$date_tranche) {
    $return = 0;
    $cilien_meta = cilien_tableau_meta();
    
    if (isset($cilien_meta[$objet])){
        if (isset($cilien_meta[$objet]['date_tranche']) AND $cilien_meta[$objet]['date_tranche']==$date_tranche){
            if (isset($cilien_meta[$objet]['id_tranche'])){
                $return = $cilien_meta[$objet]['id_tranche'];
            }
        }
    }
    
    return $return;
}

function cilien_ecrire_date_et_id($objet,$date_tranche,$id_tranche) {
    $cilien_meta = cilien_tableau_meta();
    
    $cilien_meta[$objet] = array('date_tranche'=>$date_tranche, 'id_tranche'=>$id_tranche);
    include_spip('inc/meta');
    ecrire_meta('cilien', @serialize($cilien_meta));
    
    return true;
}

function cilien_tableau_meta() {	
    $return = array();

    if (isset($GLOBALS['meta']['cilien'])) {
        $cilien_meta = @unserialize($GLOBALS['meta']['cilien']);

        if (is_array($cilien_meta))
                $return = $cilien_meta;
    }
    return $return;
}


?>